/*
 * Hangeul Regular Expression for Python.
 *
 * Copyright 2004 (C), Joon-Cheol Park
 * jooncheol@gmail.com
 * */
#include "Python.h"

#include <stdio.h>
#include <unistd.h>

#include "string.h"     
#include "malloc.h"     

#include <hre/hre.h>

static PyObject *_hre_error;

// User defined Object
typedef struct
{
	PyObject_HEAD

	// PY_HRE's member 
	hre_t 	hretype;
} PY_HRE;

extern PyTypeObject PY_HRE_Type;

static PyObject *_compile(PY_HRE *self, PyObject *args)
{ 
	PY_HRE *hreObject;
	char *exp;
	hre_option_t option;

	if(!PyArg_ParseTuple(args,"s",&exp)) {
		PyErr_SetString(_hre_error,"Usage: compile(expression)");
		return NULL;
	}


	hreObject = PyObject_NEW(PY_HRE, &PY_HRE_Type);
	if(hreObject == NULL) {
		PyErr_SetString(_hre_error,"Fail to create PY_HRE Object");
		return NULL;
	}

	hreObject->hretype=hre_compile(exp, option);
	if(!hreObject->hretype) {
		PyErr_SetString(_hre_error,"Invalid expression");
	}

	return (PyObject *)hreObject;
}



static PyMethodDef _hre_methods[] = {
    {"compile", (PyCFunction) _compile, METH_VARARGS, NULL},
    {NULL,      NULL, 0, NULL } 
};

void init_hre(void)
{
    PyObject *m, *d;

    m = Py_InitModule("_hre", _hre_methods);
    d = PyModule_GetDict(m);
    _hre_error = PyErr_NewException("_hre.error", NULL, NULL);
    PyDict_SetItemString(d, "error", _hre_error);
} 


// hre's member function
static PyObject *_hre_search(PY_HRE *self, PyObject *args)
{
	char *str;
	if(!PyArg_ParseTuple(args,"s",&str)) {
		PyErr_SetString(_hre_error,"Usage: search(str)");
		return NULL;
	}

	return Py_BuildValue("i", hre_search(self->hretype, str));
}


// PY_HRE methods
static PyMethodDef PY_HRE_methods[] = {
	{"search", (PyCFunction)_hre_search, METH_VARARGS},
	{NULL,NULL}
};

// PY_HRE dealloc
static void PY_HRE_dealloc(PY_HRE *self)
{
	hre_free(self->hretype);
	PyMem_Free((char *) self);
}

// PY_HRE getattr
static PyObject * PY_HRE_getattr(PY_HRE *self, char *name)
{
	PyObject *res;
	res = Py_FindMethod(PY_HRE_methods, (PyObject *)self, name);
	if(res != NULL)
		return res;
	PyErr_Clear();
	PyErr_SetString(_hre_error,"UnKnown method");
	return NULL;
}

// PY_HRE repr
static PyObject * PY_HRE_repr(PY_HRE *self)
{
	char buf[300];
	sprintf(buf,"<Class Hre at %lx>",(long)self);
	return PyString_FromString(buf);
}



// PY_HRE Type
PyTypeObject PY_HRE_Type = {
	#ifndef MS_WIN32
		PyObject_HEAD_INIT(&PyType_Type)
	#else
		PyObject_HEAD_INIT(NULL)
	#endif
	0,
	"Foo Object",
	sizeof(PY_HRE),
	0,
	(destructor)PY_HRE_dealloc,
	0,
	(getattrfunc)PY_HRE_getattr,
	0,
	0,
	(reprfunc)PY_HRE_repr,
};
